@@ -55,8 +55,10 @@ print_size(unsigned long size)
 	static char	unit[] = "-kMG";
 	static char	buffer[64];
 	unsigned int	power = 0;
+	/* allow size to be divided 3 times at most (k,M,G) */
+	unsigned int	max_loops = 3;
 
-	while (size && !(size % 1024) && power < sizeof(unit)) {
+	while (size && !(size % 1024) && power < max_loops) {
 		size /= 1024;
 		power++;
 	}
@@ -134,13 +136,16 @@ parse_timeout(const char *arg)
 		case '\0':
 			ret += v;
 			break;
-		case 'd':
+		case 'd':		/* days */
 			v *= 24;
-		case 'h':
+			/* fallthru */
+		case 'h':		/* hours */
 			v *= 60;
-		case 'm':
+			/* fallthru */
+		case 'm':		/* minutes */
 			v *= 60;
-		case 's':
+			/* fallthru */
+		case 's':		/* seconds */
 			ret += v;
 			++s;
 			break;
